=== modified file 'NEWS'
--- NEWS	2007-07-15 16:02:01 +0000
+++ NEWS	2007-07-04 16:57:07 +0000
@@ -1,3 +1,11 @@
+0.19.0  UNRELEASED
+
+ FEATURES
+
+  * Add support for customizing colors in the diff window via ~/.colordiffrc.
+    See README file for details. (Adeodato Simó)
+
+
 0.18.0  2007-07-14
 
  FEATURES

=== modified file 'README'
--- README	2007-07-03 22:19:26 +0000
+++ README	2007-07-04 16:57:07 +0000
@@ -87,6 +87,21 @@
 lines 2 hours old may be orange. Highlighting can be turned off with --plain
 if overwhelming.
 
+Usage - diff window
+-------------------
+
+If you have the gtksourceview python bindings installed, the diff window
+will have syntax highlighting. The colors will come from the ~/.colordiffrc
+file; since that file may be written for a dark background environment,
+the file ~/.colordiffrc.bzr-gtk can be used to override some values.
+
+Because gtksourceview is more configurable that colordiff, in addition
+to the 'oldtext', 'newtext', and 'diffstuff' keys, the following keys
+are also accepted: 'location', 'file', 'specialcase'.
+
+Colors can be specified with names (valid names are those in the
+/usr/share/X11/rgb.txt file), or with a #RRGGBB notation.
+
 Install on Windows
 ------------------
 

=== modified file 'diff.py'
--- diff.py	2007-02-03 13:01:12 +0000
+++ diff.py	2007-07-04 16:57:07 +0000
@@ -14,6 +14,8 @@
 
 import gtk
 import pango
+import os
+import re
 import sys
 
 try:
@@ -26,6 +28,7 @@
 
 from bzrlib.diff import show_diff_trees
 from bzrlib.errors import NoSuchFile
+from bzrlib.trace import warning
 
 
 class DiffWindow(gtk.Window):
@@ -92,6 +95,7 @@
             self.buffer = gtksourceview.SourceBuffer()
             slm = gtksourceview.SourceLanguagesManager()
             gsl = slm.get_language_from_mime_type("text/x-patch")
+            self.apply_colordiffrc(gsl)
             self.buffer.set_language(gsl)
             self.buffer.set_highlight(True)
 
@@ -167,3 +171,73 @@
         s = StringIO()
         show_diff_trees(self.parent_tree, self.rev_tree, s, specific_files)
         self.buffer.set_text(s.getvalue().decode(sys.getdefaultencoding(), 'replace'))
+
+    @staticmethod
+    def apply_colordiffrc(lang):
+        """Set style colors in lang to that specified in colordiff config file.
+
+        Both ~/.colordiffrc and ~/.colordiffrc.bzr-gtk are read.
+
+        :param lang: a gtksourceview.SourceLanguage object.
+        """
+        def parse_colordiffrc(fileobj):
+            """Parse fileobj as a colordiff configuration file.
+            
+            :return: A dict with the key -> value pairs.
+            """
+            colors = {}
+            for line in fileobj:
+                if re.match(r'^\s*#', line):
+                    continue
+                key, val = line.split('=')
+                colors[key.strip()] = val.strip()
+            return colors
+
+        colors = {}
+
+        for f in ('~/.colordiffrc', '~/.colordiffrc.bzr-gtk'):
+            f = os.path.expanduser(f)
+            if os.path.exists(f):
+                try:
+                    f = file(f)
+                except IOError, e:
+                    warning('could not open file %s: %s' % (f, str(e)))
+                else:
+                    colors.update(parse_colordiffrc(f))
+                    f.close()
+
+        if not colors:
+            # ~/.colordiffrc does not exist
+            return
+
+        mapping = {
+                # map GtkSourceView tags to colordiff names
+                # since GSV is richer, accept new names for extra bits,
+                # defaulting to old names if they're not present
+                'Added@32@line': ['newtext'],
+                'Removed@32@line': ['oldtext'],
+                'Location': ['location', 'diffstuff'],
+                'Diff@32@file': ['file', 'diffstuff'],
+                'Special@32@case': ['specialcase', 'diffstuff'],
+        }
+
+        for tag in lang.get_tags():
+            tag_id = tag.get_id()
+            keys = mapping.get(tag_id, [])
+            color = None
+
+            for key in keys:
+                color = colors.get(key, None)
+                if color is not None:
+                    break
+
+            if color is None:
+                continue
+
+            style = gtksourceview.SourceTagStyle()
+            try:
+                style.foreground = gtk.gdk.color_parse(color)
+            except ValueError:
+                warning('not a valid color: %s' % color)
+            else:
+                lang.set_tag_style(tag_id, style)

